#include <iostream>
#include "tableau.hpp"


const int N = 4; // Taille des matrices considerees



/*
  On va calculer le determinant de maniere recursive :
  - on copie la matrice dans une autre zone memoire pour pouvoir
    la modifier sans perdre la premiere matrice
  - par un pivot de Gauss, on ramene une matrice carre
    |****|
    |****|
    |****|
    |****|
    a une matrice de la forme
    |****|
    |****|
    |****|
    |000*|
  - le determinant est alors le coefficient en bas a droite
    multiplie par le determinant du mineur (n-1)x(n-1) en
	haut a gauche.

  Par consequent, on organise le calcul du determinant comme suit :
  - la fonction "det" copie la matrice dans une nouvelle zone memoire 
  - une fonction auxilliaire det_aux calcule le determinant d'un mineur,
    en s'autorisant a modifier la matrice passee en argument.
*/

double det_aux(double **M, int n);

double det(double **M, int n)
{
  double **M_copie = new double*[n];
  for (int i=0; i<n; i++)
	M_copie[i] = new double[n];

  for (int i=0; i<n; i++)
	for (int j=0; j<n; j++)
	  M_copie[i][j] = M[i][j];
  
  double d = det_aux(M_copie,n);

  for (int i=0; i<n; i++)
	delete [] M_copie[i];

  delete M_copie;
  
  return d;
}





double det_aux(double **M, int n)
{
  if (n==1)
	return M[0][0];
  
  int i = i_maximum_abs(M[n-1], n);
  // On cherche le maximum (en valeur absolue) de la derniere ligne
  
  if (M[n-1][i]==0)
	// Si la derniere ligne est nulle, le determinant est nul.
	return 0;

  /*
	Si le maximum n'est pas au bout de la ligne, on echange deux
	colonnes pour l'y mettre (en changeant le signe d'une des
	colonnes pour conserver le meme determinant). En effet,
	l'algorithme est plus stable si c'est la plus grande valeur
	qui sert de pivot.
  */
  if (i!=n-1) 
	{
	  double tampon;
	  for (int j=0; j<n; j++)
		{
		  tampon = M[j][n-1];
		  M[j][n-1] = -M[j][i];
		  M[j][i] = tampon;
		}
	}
 
  // On fait le pivot de Gauss
  for (int j=0; j<n-1; j++)
	for (int i=0; i<n; i++)
	  M[i][j] -= M[i][n-1] * M[n-1][j] / M[n-1][n-1];

  /* On s'est ramene au cas d'une matrice de la forme
	 | ****** |
	 | ****** |
	 | 00000* |	 
   */
  return M[n-1][n-1] * det_aux(M, n-1);
}


int main(void)
{
  double **M = new double*[N];
  for (int i=0; i<N; i++)
	{
	  M[i] = new double[N];
	  remplit_tableau(M[i], N); // On remplit la ieme ligne du tableau
	}

  std::cout << "Matrice dont les coefficients sont tires selon des lois uniformes sur [0,1] :\n";
  for (int i=0; i<N; i++)
	{
	  for (int j=0; j<N; j++)
		std::cout << M[i][j] << "  ";
	  std::cout << std::endl;
	}
  
  std::cout << "\nSon determinant vaut " << det(M,N) << std::endl;
  return 0;

}
